﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;
using System.Diagnostics;
using AForge.Video;
using AForge.Controls;
using AForge.Video.VFW;
using Microsoft.Win32;//used for getting sleep mode event


//General Note: Content of FFmpeg_LGPL_Files folder is copied to output directory you during post build project events

namespace Clarus_Med_Scope_Viewer
{
    public partial class frmMain : Form
    {
        private ScopeViewController _scopeViewControl;
        //private string version = "0.0.7";
        //private string version = "0.0.8"; //had done some testing for different video writers, added frame delay in video writer
        //private string version = "0.0.9"; //added useLegacyV2RuntimeActivationPolicy="true" to app.config file
        private string version = "1.0.0"; //first released version.  Code cleanup and commenting

        //v0.0.9 branding to control icons and logos depending on brand type (Clarus or Healthmark)
        //see instructions for changing branding
        enum branding {Clarus,Healthmark,}
        branding _brand = branding.Clarus;

        private bool _formLoaded = false;

        private Logging _debugLogging;
        private bool _debugLoggingEnabled = false;


        public frmMain()
        {
            InitializeComponent();
        }



        private void frmMain_Load(object sender, EventArgs e)
        {
            //v0.0.9 moved everything into an Initialize method so that it can be called after waking from Sleep mode
            setupBranding();
            Initialize();
        }
        
        private void setupBranding()
        {
            if (_brand == branding.Clarus)
            {
                //do nothing, this is the default setup

            }
            else if (_brand == branding.Healthmark)
            {
                try
                {
                    this.Text = "Healthmark Scope Viewer";
                    pbLogo.Load("healthMark_logo.png");
                    this.BackColor = Color.White;
                }
                catch
                {
                    MessageBox.Show("Cannot display Healthmark branding");
                }
            }
        }

        private void Initialize()
        {
            //turn on debug logging if enabled in the settings
            _debugLoggingEnabled = Properties.Settings.Default.DebugLogging;

            //v0.0.9 get the list of camera moniker strings that shoudl be excluded from the remote capture feature
            string cameraExclMonikerString = Properties.Settings.Default.CamMonikerExclusions;//v0.0.9

            if (_debugLoggingEnabled)
            {
                if(_brand == branding.Healthmark)
                    _debugLogging = new Logging("HealthmarkLog");
                else
                    _debugLogging = new Logging("ClarusMed");
            }

            //create ScopeViewController and pass in the UI video player
            try
            {
                if (_debugLoggingEnabled)
                    _scopeViewControl = new ScopeViewController(vidPlayerMain, _debugLogging, cameraExclMonikerString);
                else
                    _scopeViewControl = new ScopeViewController(vidPlayerMain, cameraExclMonikerString);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }


            this.Text = this.Text + " v" + version;

            labClock.Text = DateTime.Now.ToString();

            string lastVideoDeviceName = "";
            int lastWidth = 0;
            int lastHeight = 0;
            try
            {
                lastVideoDeviceName = Properties.Settings.Default.VideoDeviceName;
                lastWidth = Properties.Settings.Default.VideoWidth;
                lastHeight = Properties.Settings.Default.VideoHeight;
            }
            catch
            {
                MessageBox.Show("Could not load settings");
            }

            bool deviceLoaded = _scopeViewControl.loadVideoDeviceOnStartup(lastVideoDeviceName, lastHeight, lastWidth);
            if (deviceLoaded == false)
                SetControlItems(false);
            else
                SetControlItems(true);

            //v0.0.9 - get the last saved files location.  If it cannot be found then create default
            string folderPath = Properties.Settings.Default.FileLoc;
            if (folderPath == string.Empty)
            {
                if(_brand == branding.Healthmark)
                {
                    folderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\Healthmark\\";
                    if (Directory.Exists(folderPath) == false)
                    {
                        Directory.CreateDirectory(folderPath);
                    }
                }
                else//use Clarus as default
                {
                    folderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\ClarusMedical\\";
                    if (Directory.Exists(folderPath) == false)
                    {
                        Directory.CreateDirectory(folderPath);
                    }
                }

            }

            _scopeViewControl.FolderPath = folderPath;
            tbFileLoc.Text = folderPath;

            //subscribe to the video status changed event from the "ScopeViewControl" object
            _scopeViewControl.VideoStatusChanged += new EventHandler<VideoRecordingStatusArgs>(_scopeViewControl_VideoStatusChanged);

            //subscribe to error timer status changed event from the ScopeViewControl object
            _scopeViewControl.TimerErrorChanged += new EventHandler<TimerErrorArgs>(_scopeViewControl_TimerErrorChanged);

            updateFilesList();

            resizeVideoPlayer();



            //recover "Study Name" from last time
            try
            {
                tbStudyName.Text = Properties.Settings.Default.StudyName;
                _scopeViewControl.StudyName = Properties.Settings.Default.StudyName;
            }
            catch
            {
                MessageBox.Show("Could not load settings file");
            }
            _formLoaded = true;
        }


        //handle timer error change event from ScopeViewController Object
        public void _scopeViewControl_TimerErrorChanged(object sender, TimerErrorArgs e)
        {
            if (this.InvokeRequired)
            {
                //Invoke a new timer error event if passed in froma  different thread
                Invoke(new Action<object, TimerErrorArgs>(_scopeViewControl_TimerErrorChanged), sender, e);
                return;
            }
            else
            {
                if (e.ErrorStatus == TimerErrorArgs.errorStatusType.Ok)
                {
                    //if no error hide the loading panel
                    panelLoading.Visible = false;
                }
                else if (e.ErrorStatus == TimerErrorArgs.errorStatusType.Loading)
                {
                    labVidStatus.Text = "Processing Please Wait...";
                    pbLoading.Image = Properties.Resources.Loading;
                    pbLoading.Invalidate();
                    panelLoading.Visible = true;
                }
                else if (e.ErrorStatus == TimerErrorArgs.errorStatusType.Error)
                {
                    labVidStatus.Text = "Video Device Lost";
                    pbLoading.Image = Properties.Resources._109_AllAnnotations_Error_256x256;
                    pbLoading.Invalidate();
                    panelLoading.Visible = true;

                }
            }
        }

        //handle video status change event from ScopeViewControllerObject
        public void _scopeViewControl_VideoStatusChanged(object sender, VideoRecordingStatusArgs e)
        {
            //Invoke a new Video Status event if passed in from a different thread
            if (InvokeRequired)
            {
                BeginInvoke(new EventHandler<VideoRecordingStatusArgs>(_scopeViewControl_VideoStatusChanged), sender, e);
                return;
            }
            else
            {
                if (e.CapEventType == VideoRecordingStatusArgs.captureEventType.VideoStarted)
                {
                    enableDisableUiControls(false);
                    bCapture.Text = "Stop Video";
                    labStatus.Text = "Recording...";
                    bCapture.BackColor = Color.Red;
                    labStatus.Visible = true;
                }
                else if (e.CapEventType == VideoRecordingStatusArgs.captureEventType.VideoStopped)
                {
                    enableDisableUiControls(true);
                    bCapture.Text = "Capture";
                    bCapture.BackColor = Control.DefaultBackColor;
                    labStatus.Visible = false;
                    timNotificationUpdate.Start();
                }
                else if (e.CapEventType == VideoRecordingStatusArgs.captureEventType.PictureCaptured)
                {
                    _scopeViewControl.CaptureEventActive = true;//v0.0.9
                    labStatus.Text = "Picture Captured";
                    labStatus.Visible = true;
                    timNotificationUpdate.Start();
                }

            }
        }

        private void timClock_Tick(object sender, EventArgs e)
        {
            //update the lable that display the date and time
            labClock.Text = DateTime.Now.ToString();
        }

        private void SetControlItems(bool controlEnabled)
        {
            bCapture.Enabled = controlEnabled;
            rbPicture.Enabled = controlEnabled;
            rbVideo.Enabled = controlEnabled;
            bRefCapture.Enabled = controlEnabled;
            bRefOpen.Enabled = controlEnabled;
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            //SystemEvents.PowerModeChanged -= SystemEvents_PowerModeChanged;
            _scopeViewControl.Close();
        }

        private void rbPicture_CheckedChanged(object sender, EventArgs e)
        {
            if (rbPicture.Checked)
            {
                _scopeViewControl.CaptureMode = ScopeViewController.CaptureModeType.Picture;
            }
        }

        private void rbVideo_CheckedChanged(object sender, EventArgs e)
        {
            if (rbVideo.Checked)
            {
                _scopeViewControl.CaptureMode = ScopeViewController.CaptureModeType.Video;
            }
        }

        private void bCapture_Click(object sender, EventArgs e)
        {
            capture();
        }

        private void capture()
        {
            
            bool videoRecInProcess = _scopeViewControl.VideoRecordingNow;
            var capMode = _scopeViewControl.CaptureMode;

            if (capMode == ScopeViewController.CaptureModeType.Video)
            {
                if (videoRecInProcess == false)
                {
                    bool startOk = _scopeViewControl.startCapture();
                    if (startOk)
                    {
                        //hande changing UI elements from Video Status event
                    }
                    else
                        MessageBox.Show("Could not start capture");
                }
                else if (videoRecInProcess == true)
                {
                    _scopeViewControl.stopCapture();
                }
            }
            else if (capMode == ScopeViewController.CaptureModeType.Picture)
            {
                bool capStartOk = _scopeViewControl.startCapture();
            }
        }

        private void enableDisableUiControls(bool enable)
        {
            rbPicture.Enabled = enable;
            rbVideo.Enabled = enable;
            bSettings.Enabled = enable;
            bSetFileLoc.Enabled = enable;
            Application.DoEvents();
        }

        private void bSetFileLoc_Click(object sender, EventArgs e)
        {
            if (fbdSetFileLoc.ShowDialog() == DialogResult.OK)
            {
                _scopeViewControl.FolderPath = fbdSetFileLoc.SelectedPath + "\\";
                tbFileLoc.Text = _scopeViewControl.FolderPath;
                updateFilesList();

                //v0.0.9 save file location for next time
                Properties.Settings.Default.FileLoc = fbdSetFileLoc.SelectedPath + "\\";
                Properties.Settings.Default.Save();
            }
        }


        private void bRefOpen_Click(object sender, EventArgs e)
        {
            ofdReferenceImage.InitialDirectory = _scopeViewControl.FolderPath;
            if (ofdReferenceImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    pbReferanceImage.Image = new Bitmap(ofdReferenceImage.FileName);
                }
                catch
                {
                    MessageBox.Show("Could not load reference image");
                }
            }
        }

        //if the user selects capture reference image, save the image to a RefernceImages file and disply on UI
        private void bRefCapture_Click(object sender, EventArgs e)
        {
            try
            {
                Bitmap bitMap = vidPlayerMain.GetCurrentVideoFrame();
                pbReferanceImage.Image = bitMap;

                //v0.0.9 create a subfolder if nesissary and save reference image there
                string fileName = DateTime.Now.ToString("yyyyMMddHHmmssmmFFF") + "_" + tbStudyName.Text + "_Ref.bmp";
                string folderPath = tbFileLoc.Text + "\\ReferenceImages\\";

                if (Directory.Exists(folderPath) == false)
                {
                    Directory.CreateDirectory(folderPath);
                }

                bitMap.Save(folderPath + fileName);

            }
            catch
            {
                MessageBox.Show("Could not capture reference image");
            }

        }

        //double clicking the file location texbox will open up the directory
        private void tbFileLoc_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                Process.Start(tbFileLoc.Text);
            }
            catch
            {
                MessageBox.Show("Could not open file location");
            }
        }

        private void bSettings_Click(object sender, EventArgs e)
        {
            frmSettings settingsForm = new frmSettings(_scopeViewControl);
            settingsForm.ShowDialog();
            if(settingsForm.FormAccepted)
            {
                //save settings for next time
                try
                {
                    Properties.Settings.Default.VideoDeviceName = settingsForm.SelectedVideoDeviceName;
                    Properties.Settings.Default.VideoHeight = settingsForm.SelectedFrameHeight;
                    Properties.Settings.Default.VideoWidth = settingsForm.SelectedFrameWidth;
                    Properties.Settings.Default.Save();
                }
                catch
                {
                    MessageBox.Show("Could not save settings");
                }

                string remoteCaptureExlusions = Properties.Settings.Default.CamMonikerExclusions;

                bool vidDeviceChangedOk = _scopeViewControl.changeVideoDevice(settingsForm.videoDevice, 
                    settingsForm.SelectedVideoDeviceName,settingsForm.SelectedFrameHeight,settingsForm.SelectedFrameWidth, remoteCaptureExlusions);

                if (vidDeviceChangedOk == false)
                {
                    MessageBox.Show("Could not change video device");
                }
                else
                {
                    resizeVideoPlayer();
                }


            }
            else if(settingsForm.RemCaptExclusionsSet)
            {
                string remoteCaptureExlusions = Properties.Settings.Default.CamMonikerExclusions;
                _scopeViewControl.setRemoteCapture(remoteCaptureExlusions);
            }
        }

        //display form for user to enter a new study name
        private void bSetStudyName_Click(object sender, EventArgs e)
        {
            SetStudyName setStudyNameForm = new SetStudyName();
            setStudyNameForm.StudyNameText = tbStudyName.Text;
            setStudyNameForm.ShowDialog();
            if (setStudyNameForm.FormAccepted)
            {
                _scopeViewControl.StudyName = setStudyNameForm.StudyNameText;
                tbStudyName.Text = setStudyNameForm.StudyNameText;
                Properties.Settings.Default.StudyName = setStudyNameForm.StudyNameText;
                Properties.Settings.Default.Save();
            }
            Properties.Settings.Default.StudyName = setStudyNameForm.StudyNameText;

        }

        //update listbox containing all of the captured files in the working directory
        public void updateFilesList()
        {
            try
            {
                lbFiles.Items.Clear();
                string directory = _scopeViewControl.FolderPath;
                string[] files = Directory.GetFiles(directory);

                for (int i = files.Length - 1; i >= 0; i--)
                {
                    if (Path.GetExtension(files[i]).ToUpper() == ".BMP" || Path.GetExtension(files[i]).ToUpper() == ".AVI")
                    {
                        lbFiles.Items.Add(Path.GetFileName(files[i]));
                    }
                }
            }
            catch
            {
                MessageBox.Show("Could not update files list");
            }

        }

        //Timer to display video captured label for a period of time after capture
        private void timNotificationUpdate_Tick(object sender, EventArgs e)
        {
            timNotificationUpdate.Stop();
            updateFilesList();
            
            labStatus.Visible = false;

            _scopeViewControl.CaptureEventActive = false;//v0.0.9
        }

        //handle layout of the video player
        private void frmMain_SizeChanged(object sender, EventArgs e)
        {
            if(_formLoaded)
            {
                resizeVideoPlayer();
                moveLoadingPanel();
            }
        }

        private void moveLoadingPanel()
        {
            panelLoading.Top = vidPlayerMain.Top + (vidPlayerMain.Height / 2) - panelLoading.Height / 2;
            panelLoading.Left = vidPlayerMain.Left + (vidPlayerMain.Width / 2) - panelLoading.Width / 2;
        }

        private void resizeVideoPlayer()
        {
            //handle the size of the video player control
            if (vidPlayerMain != null && _scopeViewControl != null)
            {
                //set the width as wide as possible
                if(this.Width > 384)
                    vidPlayerMain.Width = this.Width - 384;


                vidPlayerMain.Height = Convert.ToInt32(vidPlayerMain.Width * _scopeViewControl.VidWidthFactor);

                //if the height exceeds the allowable space, shrink the height and width
                if (this.Height - vidPlayerMain.Height < 150)
                {
                    vidPlayerMain.Height = this.Height - 150;
                    vidPlayerMain.Width = Convert.ToInt32(vidPlayerMain.Height / _scopeViewControl.VidWidthFactor);
                }

                //center the video
                if (this.Width > 384)
                {
                    vidPlayerMain.Left = (this.Width - 290 - vidPlayerMain.Width) / 2 + 270;
                }
                if(this.Height > 150)
                {
                    vidPlayerMain.Top = (this.Height - 100 - vidPlayerMain.Height) / 2;
                }
            }
        }

        //when a file is double clicked from the files listbox, attempt to open the file using Process.Start
        private void lbFiles_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                string directory = _scopeViewControl.FolderPath;
                string filePath = directory + lbFiles.SelectedItem.ToString();
                if (File.Exists(filePath))
                {
                    Process.Start(filePath);
                }
                else
                {
                    MessageBox.Show("Could not find selected file");
                }
            }
            catch
            {
                MessageBox.Show("Could not open selected file");
            }
        }

        private void pbLogo_Click(object sender, EventArgs e)
        {
            //for later use, can show about box when clicking on the logo
            //AboutBox aboutBox = new AboutBox();
            //aboutBox.ShowDialog();
        }

        //capture the spacebar press event and use it to trigger a capture
        private void frmMain_KeyPress(object sender, KeyPressEventArgs e)
        {

            if(e.KeyChar.ToString() == " ")
            {
                capture();
            }
        }

        //when a key is down, change the focus to and element that wont be effected
        private void frmMain_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.KeyCode == Keys.Space)
            {
                label3.Focus();
            }
        }


    }
}
