﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AForge.Video;
using AForge.Video.DirectShow;

namespace Clarus_Med_Scope_Viewer
{
    public partial class frmSettings : Form
    {
        private FilterInfoCollection videoDevices;
        public VideoCaptureDevice videoDevice;
        private VideoCapabilities[] videoCapabilities;
        private VideoCapabilities[] snapshotCapabilities;

        public bool FormAccepted = false;
        public bool RemCaptExclusionsSet = false;
        public string SelectedVideoDeviceName;
        public int SelectedFrameHeight;
        public int SelectedFrameWidth;

        public List<CameraSettingData> _camSettList = new List<CameraSettingData>();

        private bool _intitalStartup = true;

        private ScopeViewController _scopeViewController;
        public frmSettings(ScopeViewController ScopeViewControllerObject)
        {
            InitializeComponent();
            _scopeViewController = ScopeViewControllerObject;
        }

        private void frmSettings_Load(object sender, EventArgs e)
        {
            // enumerate video devices
            videoDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);

            int currentVidDeviceIndex = 0;
            bool curVidDeviceFound = false;
            if (videoDevices.Count != 0)
            {
                // add all devices to combo
                for (int i = 0; i < videoDevices.Count; i++)
                {
                    //v0.0.9 add camera data so that it can be used in exclusions list -added 12/20/17
                    CameraSettingData camSetData = new CameraSettingData();
                    camSetData.CameraName = videoDevices[i].Name;
                    camSetData.Moniker = videoDevices[i].MonikerString;
                    _camSettList.Add(camSetData);

                    lbVideoDeviecs.Items.Add(videoDevices[i].Name);
                    if (videoDevices[i].Name == _scopeViewController.VideoDeviceName)
                    {
                        currentVidDeviceIndex = i;
                        curVidDeviceFound = true;
                    }
                }
                lbResolution.Enabled = true;
            }
            else
            {
                lbVideoDeviecs.Items.Add("No video devices found");
            }

            if(curVidDeviceFound)
            {
                lbVideoDeviecs.SelectedIndex = currentVidDeviceIndex;
            }
            else
            {
                lbVideoDeviecs.SelectedIndex = 0;
            }

        }

        private void lbVideoDeviecs_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(videoDevices.Count != 0)
            {
                videoDevice = new VideoCaptureDevice(videoDevices[lbVideoDeviecs.SelectedIndex].MonikerString);
                EnumeratedSupportedFrameSizes(videoDevice);
            }
        }

        // Collect supported video
        private void EnumeratedSupportedFrameSizes(VideoCaptureDevice videoDevice)
        {
            //this.Cursor = Cursors.WaitCursor;

            lbResolution.Items.Clear();
            //snapshotResolutionsCombo.Items.Clear();

            try
            {
                videoCapabilities = videoDevice.VideoCapabilities;
                snapshotCapabilities = videoDevice.SnapshotCapabilities;

                int curResolutionIndex = 0;
                bool curResolutionFound = false;

                for(int i = 0; i< videoCapabilities.Length; i++)
                {


                    lbResolution.Items.Add(string.Format("{0} x {1}",
                        videoCapabilities[i].FrameSize.Width, videoCapabilities[i].FrameSize.Height));

                    if (videoCapabilities[i].FrameSize.Height == _scopeViewController.VideoDeviceHeight &&
                        videoCapabilities[i].FrameSize.Width == _scopeViewController.VideoDeviceWidth)
                    {
                        curResolutionIndex = i;
                        curResolutionFound = true;
                    }
                }


                if (videoCapabilities.Length == 0)
                {
                    lbResolution.Items.Add("Not supported");
                }

                if(_intitalStartup)
                {
                    _intitalStartup = false;
                    if(curResolutionFound)
                    {
                        lbResolution.SelectedIndex = curResolutionIndex;
                    }
                    else
                    {
                        lbResolution.SelectedIndex = 0;
                    }
                }
                else
                {
                    lbResolution.SelectedIndex = 0;
                }
                
                bOk.Enabled = true;
            }
            finally
            {
                //this.Cursor = Cursors.Default;
            }
        }

        private void bOk_Click(object sender, EventArgs e)
        {
            SelectedVideoDeviceName = videoDevices[lbVideoDeviecs.SelectedIndex].Name;
            videoDevice.VideoResolution = videoCapabilities[lbResolution.SelectedIndex];
            SelectedFrameHeight = videoDevice.VideoResolution.FrameSize.Height;
            SelectedFrameWidth = videoDevice.VideoResolution.FrameSize.Width;
            FormAccepted = true;
            this.Close();
        }

        private void bCancel_Click(object sender, EventArgs e)
        {
            FormAccepted = false;
            this.Close();
        }

        private void bAdvanced_Click(object sender, EventArgs e)
        {
            frmExclusions exclForm = new frmExclusions(_camSettList);
            exclForm.ShowDialog();
            if (exclForm.FormExcepted)
            {
                RemCaptExclusionsSet = true;
                try
                {
                    string exclusionsString = "";
                    foreach (var excl in exclForm.CamExclList)
                    {
                        exclusionsString = exclusionsString + excl + ",";
                    }
                    Properties.Settings.Default.CamMonikerExclusions = exclusionsString;
                    Properties.Settings.Default.Save();
                }
                catch
                {
                    MessageBox.Show("Could not save remote capture exclusion settings");
                }
            }
        }
    }
}
